@@ -1,7 +1,7 @@
 """History Callback"""
 import fnmatch
 import warnings
-from typing import List, Optional, Union
+from typing import List, Optional, Union, Tuple
 
 import matplotlib.pyplot as plt
 import numpy as np
@@ -35,8 +35,10 @@ class History(Logger):
              features: Union[None, int, str, List[str], List[List[Optional[str]]]] = '/',
              excluded: Optional[List[str]] = None,
              orient_rows: bool = False,
-             tight_layout=True,
-             figsize=(16, 9),
+             title: Optional[str] = None,
+             titles_pad: int = 20,
+             tight_layout: bool = True,
+             figsize: Optional[Tuple[int, int]] = (16, 9),
              **plt_kwargs):
         """Plots the training information which were previously collected in a dataframe.
 
@@ -62,6 +64,12 @@ class History(Logger):
             when 'orient_rows' is set to True the 'num_subplots' indicates the number of rows while when 'orient_rows'
             is set to False the 'num_subplots' indicates the number of columns.
 
+        :param title:
+            The (optional) title of the whole plot.
+
+        :param titles_pad:
+            The padding of each subplot title to avoid overlapping with labels.
+
         :param tight_layout:
             Matplotlib `figure()` argument.
 
@@ -130,8 +138,11 @@ class History(Logger):
                 x, y = self._history.index, self._history[column]
                 plt.plot(x, y)
                 plt.scatter(x, y)
-                ax.set(title=column, xlabel='', ylabel='')
+                ax.set(xlabel='', ylabel='')
+                ax.set_title(column, pad=titles_pad)
                 # fix integer x_ticks based on autogenerated ones in order to avoid superimposed values
                 ticks = np.unique(ax.get_xticks().round().astype(int))
                 ax.set_xticks([t for t in ticks if t in range(x.min(), x.max() + 1)])
+        if title is not None:
+            plt.suptitle(title)
         plt.show()
