@@ -6,6 +6,7 @@ import numpy as np
 
 from moving_targets.callbacks import StatsLogger
 from moving_targets.util.errors import not_implemented_message
+from moving_targets.util.masking import mask_data, get_mask
 from moving_targets.util.scalers import Scaler
 from moving_targets.util.typing import Dataset
 
@@ -18,10 +19,14 @@ class Learner(StatsLogger):
         return {'elapsed_time'}
 
     def __init__(self,
+                 mask: Optional[float],
                  x_scaler: Union[None, Scaler, str],
                  y_scaler: Union[None, Scaler, str],
                  stats: Union[bool, List[str]]):
         """
+        :param mask:
+            The (optional) masking value used to mask the original targets.
+
         :param x_scaler:
             The (optional) scaler for the input data, or a string representing the default scaling method.
 
@@ -34,6 +39,9 @@ class Learner(StatsLogger):
         """
         super(Learner, self).__init__(stats=stats, name='Learner')
 
+        self.mask: Optional[float] = mask
+        """The (optional) masking value used to mask the original targets."""
+
         self.x_scaler: Optional[Scaler] = Scaler(default_method=x_scaler) if isinstance(x_scaler, str) else x_scaler
         """The (optional) scaler for the input data."""
 
@@ -73,6 +81,7 @@ class Learner(StatsLogger):
         :return:
             The `Learner` itself.
         """
+        x, y = mask_data(x, y, mask=get_mask(y, self.mask))
         x = x if self.x_scaler is None else self.x_scaler.fit_transform(data=x)
         y = y if self.y_scaler is None else self.y_scaler.fit_transform(data=y)
         self._fit(x=x, y=y, sample_weight=sample_weight)
